# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2019-2023, Intel Corporation
#

"""Granularity context classes and utilities"""

import os
import shutil
from enum import Enum, unique

import context as ctx
import configurator
import futils

from tools import Tools
from tools import Ndctl


class Granularity(metaclass=ctx.CtxType):
    """
    Represents file system granularity context element.

    Attributes:
        gran_detecto_arg (str): argument of gran_detecto tool
            that checks test directory for this granularity
            type compliance
        config_dir_field (str): name of the field in testconfig
            that represents the test directory with this
            granularity type
        config_force_field (str): name of the field in testconfig
            that represent the 'force' argument for this
            granularity type
        force_env (str): value of PMEM2_FORCE_GRANULARITY environment
            variable for this granularity type
        pmem_force_env (str): value for legacy PMEM_IS_PMEM_FORCE
            environment variable that corresponds to this granularity
            type

    """

    gran_detecto_arg = None
    config_dir_field = None
    config_force_field = None
    force_env = None
    pmem_force_env = None

    def __init__(self, **kwargs):
        self._env = {}
        self.tools = Tools(self._env, None)
        futils.set_kwargs_attrs(self, kwargs)
        self.config = configurator.Configurator().config
        self.dir_ = os.path.abspath(getattr(self.config,
                                            self.config_dir_field))
        self.testdir = os.path.join(self.dir_, self.tc_dirname)
        force = getattr(self.config, self.config_force_field)
        if force:
            self.env = {
                        'PMEM2_FORCE_GRANULARITY': self.force_env,

                        # PMEM2_FORCE_GRANULARITY is implemented only by
                        # libpmem2. Corresponding PMEM_IS_PMEM_FORCE variable
                        # is set to support tests for older PMDK libraries.
                        'PMEM_IS_PMEM_FORCE': self.pmem_force_env
                        }

    def setup(self, tools=None):
        if not os.path.exists(self.testdir):
            os.makedirs(self.testdir)

        check_page = tools.gran_detecto(self.testdir, self.gran_detecto_arg)
        if check_page.returncode != 0:
            msg = check_page.stdout
            detect = tools.gran_detecto(self.testdir, '-d')
            msg = '{}{}{}'.format(os.linesep, msg, detect.stdout)
            raise futils.Fail('Granularity check for {} failed: {}'
                              .format(self.testdir, msg))

    def clean(self, *args, **kwargs):
        shutil.rmtree(self.testdir, ignore_errors=True)

    @classmethod
    def testdir_defined(cls, config):
        """
        Check if test directory used by specific granularity setup
        is defined by test configuration
        """
        try:
            if not cls.config_dir_field:
                return False
            getattr(config, cls.config_dir_field)
        except futils.Skip as s:
            msg = futils.Message(config.unittest_log_level)
            msg.print_verbose(s)
            return False
        else:
            return True

    def _check_real_pmem_req_is_met(self, tc):
        req_real_pmem, _ = ctx.get_requirement(tc, 'require_real_pmem', False)
        if not req_real_pmem:
            return True

        devdaxes, _ = ctx.get_requirement(tc, 'require_devdax', None)
        # devdax cases should be dealt in devdax module
        if devdaxes is not None:
            return True

        if Ndctl().is_emulated(self.dir_):
            raise futils.Skip('skip emulated pmem')

        return True

    def _check_usc_req_is_met(self, tc):
        require_usc, _ = ctx.get_requirement(tc, 'require_usc', False)
        if not require_usc:
            return True

        basedir = self.testdir.replace(self.testdir, '')
        filepath = os.path.join(basedir, "__usc_test_file")
        f = open(filepath, 'w')
        f.close()

        check = self.tools.usc_permission_check(filepath)
        usc_available = check.returncode == 0

        os.remove(filepath)

        if not usc_available:
            raise futils.Skip('unsafe shutdown count is not available')

        return usc_available

    @classmethod
    def filter(cls, config, msg, tc):
        """
        Initialize granularity classes for the test to be run
        based on configuration and test requirements.

        Args:
            config: configuration as returned by Configurator class
            msg (Message): level based logger class instance
            tc (BaseTest): test case, from which the granularity
                requirements are obtained

        Returns:
            list of granularities on which the test should be run

        """
        req_gran, kwargs = ctx.get_requirement(tc, 'granularity', None)

        # remove granularities if respective test directories in
        # test config are not defined
        conf_defined = [c for c in config.granularity
                        if c.testdir_defined(config)]

        if req_gran == Non:
            return [Non(**kwargs), ]

        if req_gran == [_CACHELINE_OR_LESS]:
            tmp_req_gran = [Byte, CacheLine]
        elif req_gran == [_PAGE_OR_LESS]:
            tmp_req_gran = [Byte, CacheLine, Page]
        elif req_gran == [ctx.Any, ]:
            tmp_req_gran = ctx.Any.get(conf_defined)
        else:
            tmp_req_gran = req_gran

        filtered = ctx.filter_contexts(conf_defined, tmp_req_gran)

        kwargs['tc_dirname'] = tc.tc_dirname

        if len(filtered) > 1 and req_gran == [_CACHELINE_OR_LESS] or \
           req_gran == [_PAGE_OR_LESS]:

            def order_by_smallest(elem):
                ordered = [Byte, CacheLine, Page]
                return ordered.index(elem)

            # take the smallest available granularity
            filtered.sort(key=order_by_smallest)
            filtered = [filtered[0], ]

        gs = []
        for g in filtered:
            try:
                gran = g(**kwargs)
                gran._check_usc_req_is_met(tc)
                gran._check_real_pmem_req_is_met(tc)
                gs.append(gran)
            except futils.Skip as s:
                msg.print('{}: SKIP: {}'.format(tc, s))

        return gs


class Page(Granularity):
    config_dir_field = 'page_fs_dir'
    config_force_field = 'force_page'
    force_env = 'PAGE'
    pmem_force_env = '0'
    gran_detecto_arg = '-p'


class CacheLine(Granularity):
    config_dir_field = 'cacheline_fs_dir'
    config_force_field = 'force_cacheline'
    force_env = 'CACHE_LINE'
    pmem_force_env = '1'
    gran_detecto_arg = '-c'


class Byte(Granularity):
    config_dir_field = 'byte_fs_dir'
    config_force_field = 'force_byte'
    force_env = 'BYTE'
    pmem_force_env = '1'
    gran_detecto_arg = '-b'


class Non(Granularity):
    """
    No filesystem is used - granularity is irrelevant to the test.
    To ensure that the test indeed does not use any filesystem,
    accessing some fields of this class is prohibited.
    """
    explicit = True

    def __init__(self, **kwargs):
        pass

    def setup(self, tools=None):
        pass

    def cleanup(self, *args, **kwargs):
        pass

    def __getattribute__(self, name):
        if name in ('dir', ):
            raise AttributeError("'{}' attribute cannot be used if the"
                                 " test is meant not to use any "
                                 "filesystem"
                                 .format(name))
        else:
            return super().__getattribute__(name)


# special kind of granularity requirement
_CACHELINE_OR_LESS = 'cacheline_or_less'
_PAGE_OR_LESS = 'page_or_less'


@unique
class _Granularity(Enum):
    PAGE = Page
    CACHELINE = CacheLine
    BYTE = Byte
    CL_OR_LESS = _CACHELINE_OR_LESS
    PAGE_OR_LESS = _PAGE_OR_LESS
    ANY = ctx.Any


PAGE = _Granularity.PAGE
CACHELINE = _Granularity.CACHELINE
BYTE = _Granularity.BYTE
CL_OR_LESS = _Granularity.CL_OR_LESS
PAGE_OR_LESS = _Granularity.PAGE_OR_LESS
ANY = _Granularity.ANY


def require_granularity(*granularity, **kwargs):
    for g in granularity:
        if not isinstance(g, _Granularity):
            raise ValueError('selected granularity {} is invalid'
                             .format(g))

    enum_values = [g.value for g in granularity]

    def wrapped(tc):
        ctx.add_requirement(tc, 'granularity', enum_values, **kwargs)
        return tc
    return wrapped


def no_testdir(**kwargs):
    def wrapped(tc):
        ctx.add_requirement(tc, 'granularity', Non, **kwargs)
        return tc
    return wrapped


def require_usc(**kwargs):
    def wrapped(tc):
        ctx.add_requirement(tc, 'require_usc', True)
        return tc
    return wrapped


def require_real_pmem(**kwargs):
    def wrapped(tc):
        ctx.add_requirement(tc, 'require_real_pmem', True)
        return tc
    return wrapped



#!/usr/bin/env python3
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2019-2023, Intel Corporation

"""Main script for unit tests execution.

Contains the implementation of the TestRunner class, which is a main
test execution manager class and it's also used by TESTS.py scripts if run
individually.

TestRunner run_tests() method requires special attention
as it implements the fundamental test run workflow.

"""

# modules from unittest directory are visible from this script
import sys
import os
from os import path
sys.path.insert(1, path.abspath(path.join(path.dirname(__file__), 'unittest')))

# flake8 issues silenced:
# E402 - import statements not at the top of the file because of adding
# directory to path

import importlib.util as importutil  # noqa E402
import subprocess as sp  # noqa E402

import futils  # noqa E402
from consts import ROOTDIR # noqa E402
from basetest import get_testcases  # noqa E402
from configurator import Configurator  # noqa E402
from ctx_filter import CtxFilter  # noqa E402


class TestRunner:
    """
    Main script utility for managing tests execution.

    Attributes:
        testcases (list): BaseTest objects that are test
            cases to be run during execution.
        config: config object as returned by Configurator
        msg (Message) level based logger

    """
    def __init__(self, config, testcases):
        self.testcases = testcases
        self.config = config
        self._check_admin()
        self.msg = futils.Message(config.unittest_log_level)

        if self.config.test_sequence:
            # filter test cases from sequence
            self.testcases = [t for t in self.testcases
                              if t.testnum in self.config.test_sequence]

            # sort testcases so their sequence matches provided test sequence
            self.testcases.sort(key=lambda
                                tc: config.test_sequence.index(tc.testnum))

        if not self.testcases:
            sys.exit('No testcases to run found for selected configuration.')

    def _check_admin(self):
        if not self.config.enable_admin_tests:
            return

        try:
            sp.check_output(['sudo', '-n', 'true'], stderr=sp.STDOUT)
        except sp.CalledProcessError:
            sys.exit('Enabled "enable_admin_tests" requires '
                     'the non-interactive sudo (no password required to '
                     'perform the sudo command).')

    def run_tests(self):
        """Run selected testcases.

        Implementation of this method is crucial as a general
        tests execution workflow.

        Returns:
            main script exit code denoting the execution result.

        """
        ret = 0
        for tc in self.testcases:

            # TODO handle test type inside custom decorator
            if tc.test_type not in self.config.test_type:
                continue

            if self.config.test_label != '' and \
                    self.config.test_label not in tc.labels:
                continue

            if not tc.enabled:
                continue

            cf = CtxFilter(self.config, tc)

            # The 'c' context has to be initialized before the 'for' loop,
            # because cf.get_contexts() can return no value ([])
            # and in case of the 'futils.Fail' exception
            # self._test_failed(tc, c, f) will be called
            # with uninitilized value of the 'c' context.
            c = None
            try:
                for c in cf.get_contexts():
                    try:
                        t = tc()
                        if t.enabled:
                            if t.force_disabled:
                                self._test_disabled(t, c)
                                continue
                            else:
                                self.msg.print('{}: SETUP\t({}/{})'
                                               .format(t, t.test_type, c))
                                t._execute(c)
                        else:
                            continue

                    except futils.Skip as s:
                        self.msg.print('{}: {}SKIP{}: {}'.
                                       format(t, futils.Color.YELLOW,
                                              futils.Color.END, s))

                    except futils.Fail as f:
                        self._test_failed(t, c, f)
                        ret = 1
                    else:
                        self._test_passed(t)

            except futils.Skip as s:
                self.msg.print('{}: {}SKIP{}: {}'.
                               format(tc, futils.Color.YELLOW,
                                      futils.Color.END, s))
            except futils.Fail as f:
                self._test_failed(tc, c, f)
                ret = 1

        return ret

    def _test_failed(self, tc, ctx, fail):
        """Print message specific for failed test"""
        if self.config.tm and hasattr(tc, "elapsed"):
            tm = '\t[{:06.3F} s]'.format(tc.elapsed)
        else:
            tm = ''
        self.msg.print('{}: {}FAILED{}\t({}/{}) {}'
                       .format(tc, futils.Color.RED,
                               futils.Color.END, tc.test_type, ctx, tm))
        self.msg.print(fail)

        if not self.config.keep_going:
            sys.exit(1)

    def _test_passed(self, tc):
        """Print message specific for passed test"""
        if self.config.tm:
            tm = '\t[{:06.3F} s]'.format(tc.elapsed)
        else:
            tm = ''

        self.msg.print('{}: {}PASS{} {}'
                       .format(tc, futils.Color.GREEN, futils.Color.END, tm))

    def _test_disabled(self, tc, context):
        """Print message specific for disabled test"""
        self.msg.print('{}: {}DISABLED{}\t({}/{})'
                       .format(tc, futils.Color.YELLOW, futils.Color.END,
                               tc.test_type, context))


def _import_testfiles():
    """
    Traverse through "src/test" directory, find all "TESTS.py" files and
    import them as modules. Set imported module name to
    file directory path.

    Importing these files serves two purposes:
        - makes test classes residing in them visible
          and usable by the RUNTESTS script.
        - triggers basetest._TestType metaclass which initializes and
          registers them as actual test case classes - they are
          therefore available through basetest.get_testfiles() function

    """
    for root, _, files in os.walk(ROOTDIR):
        for name in files:
            if name == 'TESTS.py':
                testfile = path.join(root, name)
                module_name = path.dirname(testfile)
                spec = importutil.spec_from_file_location(module_name,
                                                          testfile)
                module = importutil.module_from_spec(spec)
                spec.loader.exec_module(module)


def main():
    _import_testfiles()
    config = Configurator().config
    testcases = get_testcases()

    if config.group:
        # filter selected groups
        testcases = [t for t in testcases
                     if path.basename(t.__module__) in config.group]

    if hasattr(config, 'list_testcases'):
        for t in testcases:
            print(t.name)
        sys.exit(0)

    runner = TestRunner(config, testcases)
    sys.exit(runner.run_tests())


if __name__ == '__main__':
    main()



# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2019-2023, Intel Corporation

"""
Functionalities for acquiring (through 'filtering' execution configuration and
test specification) set of context parameters with which the test will be run.
"""

import itertools

import builds
import futils
import granularity
import devdax

import context as ctx
import valgrind as vg
import requirements as req

CTX_TYPES = (vg.Valgrind, granularity.Granularity, devdax.DevDaxes)


class CtxFilter:
    """
    Generates contexts based on provided
    configuration and test requirements.

    Attributes:
        tc (BaseTest): test using the context
        config: configuration class
        msg (Message): level based logger
    """

    def __init__(self, config, tc):
        self.tc = tc
        self.config = config
        self.msg = futils.Message(config.unittest_log_level)

    def _get_builds(self):
        return builds.Build.filter(self.config, self.msg, self.tc)

    def get_contexts(self):
        """
        Generate a list of contexts based on configuration and
        test requirements
        """
        reqs = req.Requirements()
        if not reqs.check_if_all_requirements_are_met(self.tc):
            return []
        conf_params = self._get_configured_params()
        common_params = self._get_common_params()
        ctx_elem_names = list(conf_params.keys()) + list(common_params.keys())

        builds = self._get_builds()

        # Generate cartesian product of builds and context elements.
        # Each element of the product serves as a base for the separate
        # context in which the test is run
        ctx_params = itertools.product(builds, *conf_params.values(),
                                       *common_params.values())
        ctxs = []
        for cp in ctx_params:
            build = cp[0]

            ctx_elems = dict(zip(ctx_elem_names, cp[1:]))
            c = ctx.Context(build, **ctx_elems)
            c.cwd = self.tc.cwd
            ctxs.append(c)

        return ctxs

    def _get_configured_params(self):
        """
        Get special test parameters, like file system or Valgrind tool
        which final content depends on test requirements, and user
        configuration. This content is obtained through its
        static filter() method.
        """

        # TODO: instead of using a hard-coded CTX_TYPES list,
        # detect configured parameters based on whether they
        # implement the filter() method.
        # Configured parameter therefore becomes
        # the specific case of common parameter
        # and may be implemented ad-hoc by the test developer

        params = {}
        for param in CTX_TYPES:
            params[param.__name__.lower()] = \
                param.filter(self.config, self.msg, self.tc)
        return params

    def _get_common_params(self):
        """
        Get generic test params, added through ctx.add_params() function
        """
        return ctx.get_params(self.tc)



#!../env.py
#
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2019, Intel Corporation

import testframework as t
from testframework import granularity as g


@g.no_testdir()
class PMEM2_PERSIST(t.Test):
    test_type = t.Short

    def run(self, ctx):
        ctx.exec('pmem2_persist', self.test_case)


class TEST0(PMEM2_PERSIST):
    """test getting pmem2 persist functions"""
    test_case = "test_get_persist_funcs"


class TEST1(PMEM2_PERSIST):
    """test getting pmem2 flush functions"""
    test_case = "test_get_flush_funcs"


class TEST2(PMEM2_PERSIST):
    """test getting pmem2 drain functions"""
    test_case = "test_get_drain_funcs"



#!../env.py
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2021-2022, Intel Corporation
#

import testframework as t
from testframework import granularity as g
import futils
import re


@t.require_architectures('x86_64')
@g.require_granularity(g.ANY)
class PMEM_EADR(t.Test):
    test_type = t.Short
    envs0 = ()
    envs1 = ()

    isa = "((sse2)|(avx)|(avx512f))"
    perfbarrier = "((_wcbarrier)|(_nobarrier)|())"

    def run(self, ctx):
        for env in self.envs0:
            ctx.env[env] = '0'
        for env in self.envs1:
            ctx.env[env] = '1'

        file_path = ctx.create_holey_file(16 * t.MiB, 'testfile')
        ctx.env['PMEM_LOG_LEVEL'] = '15'
        ctx.exec('pmem_eADR_functions', self.test_case, file_path,)

        regex1 = F"{self.function_name}_mov_{self.isa}_noflush"
        regex2 = F"{self.function_name}_movnt_{self.isa}" \
                 F"_empty{self.perfbarrier}"
        regex3 = F"{self.function_name}_mov_{self.isa}_empty"
        regex = F"({regex1})|({regex2})|({regex3})"

        log_file = self.get_log_file_by_prefix("pmem")
        log_content = open(log_file).read()
        match = re.search(regex, log_content)

        if match is None:
            futils.fail(F"Failed to find any occurrence of "
                        F"{self.function_name}"
                        F" with eADR in pmem_{self.testnum}.log")


@t.require_build('debug')
class TEST0(PMEM_EADR):
    function_name = "memmove"
    envs1 = ('PMEM_NO_FLUSH',)
    test_case = 'test_eADR_memmove_256B'


@t.require_build('debug')
class TEST1(PMEM_EADR):
    function_name = "memmove"
    envs1 = ('PMEM_NO_FLUSH',)
    test_case = 'test_eADR_memmove_16MiB'


@t.require_build('debug')
class TEST2(PMEM_EADR):
    function_name = "memset"
    envs1 = ('PMEM_NO_FLUSH',)
    test_case = 'test_eADR_memset_256B'


@t.require_build('debug')
class TEST3(PMEM_EADR):
    function_name = "memset"
    envs1 = ('PMEM_NO_FLUSH',)
    test_case = 'test_eADR_memset_16MiB'



# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2018-2021, Intel Corporation


import os
import json


class MarkerParser:
    """
    Parse marker config file and command line arg provided by user
    via -x parameter.
    """

    def marker_file_parser(self, macros):
        """
        Parse markers passed by file.
        They should be in json format:
        { "MARKER_NAME"="ENGINE_TYPE" } and separated by commas.
        """
        markers = {}
        try:
            with open(macros) as config_file:
                markers = json.load(config_file)
        except json.decoder.JSONDecodeError:
            print(
                "Invalid config macros file format: ",
                macros,
                'Use: {"MARKER_NAME1"="ENGINE_TYPE1",'
                '"MARKER_NAME2"="ENGINE_TYPE2"}'
            )

        return markers

    def marker_cli_parser(self, macros):
        """
        Parse markers passed by cli.
        They should be in specific format:
        MARKER_NAME=ENGINE_TYPE and separated by commas.
        """
        try:
            markers_array = macros.split(",")
            return dict(pair.split("=") for pair in markers_array)
        except ValueError:
            print(
                "Invalid extended macros format: ",
                macros,
                "Use: MARKER_NAME1=ENGINE_TYPE1,MARKER_NAME2=ENGINE_TYPE2"
            )

    def get_markers(self, markerset):
        """
        Parse markers based on their format.
        """
        if markerset is not None:
            if os.path.exists(markerset):
                return self.marker_file_parser(markerset)
            else:
                return self.marker_cli_parser(markerset)



#!../env.py
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2020, Intel Corporation
#

import testframework as t
from testframework import granularity as g


@g.no_testdir()
class ObjCritnibMt(t.Test):
    def run(self, ctx):
        ctx.exec('obj_critnib_mt')


class TEST0(ObjCritnibMt):
    test_type = t.Medium


@t.require_valgrind_enabled('helgrind')
class TEST1(ObjCritnibMt):
    test_type = t.Long


@t.require_valgrind_enabled('drd')
class TEST2(ObjCritnibMt):
    test_type = t.Long



# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2021-2023, Intel Corporation
#
"""Unsafe shutdown utilities"""

import os
import subprocess as sp
import abc

import futils
import tools


def _get_dev_from_testdir(testdir):
    if not os.path.isdir(testdir):
        raise futils.Fail('{} is not an existing directory'
                          .format(testdir))
    try:
        out = sp.check_output(['df', testdir], stderr=sp.STDOUT,
                              universal_newlines=True)
    except sp.CalledProcessError as e:
        raise futils.Fail('df command failed: {}'.format(e.output)) from e
    else:
        # Assuming that correct 'df <testdir>' output looks as below:
        #
        # Filesystem      1K-blocks  Used Available Use% Mounted on
        # /dev/pmem0     1019003852 77856 967093676   1% /mnt/pmem0
        #
        # the device is the first word in the second line
        try:
            device = out.splitlines()[1].split()[0]
        except IndexError:
            raise futils.Fail('"df {}" output could not be parsed:\n{}'
                              .format(testdir, out))
        if not os.path.exists(device):
            raise futils.Fail('found device "{}" is not an existing file'
                              .format(device))
        return device


class USCTool(abc.ABC):
    """
    An abstract class implementing an interface necessary to serve as an
    unsafe shutdown handler tool for public UnsafeShutdown class.
    """
    @abc.abstractmethod
    def get_dev_dimms(self, dev):
        """
        Get underlying DIMMs of a provided device.
        """
        pass

    @abc.abstractmethod
    def read_usc(self, dimm):
        """
        Read unsafe shutdown count from DIMM.
        It is assumed that DIMM returned from get_dev_dimms
        may serve as a valid 'dimm' argument of this method.
        """
        pass


class UnsafeShutdown:
    """
    Utility class specifying high-level workflow for handling unsafe shutdown.
    "tool" is a class representing specific application used for reading
    and injecting unsafe shutdown. It should be a subclass of USCTool abstract
    class.
    """
    def __init__(self, tool: USCTool = None):
        if tool:
            if not isinstance(tool, USCTool):
                raise futils.Fail('{} should be a subclass of USCTool'
                                  .format(tool))
            self.tool = tool

        # set default injecting tool if no tool provided
        else:
            self.tool = NdctlUSC()

    def read(self, testdir):
        """
        Read unsafe shutdown count of provided device, which is a sum
        of unsafe shutdown counts of all underlying DIMMs
        """
        dev = _get_dev_from_testdir(testdir)
        dimms = self.get_dev_dimms(dev)
        return sum(self.read_from_dimms(*dimms))

    def read_from_dimms(self, *dimms):
        """
        Read unsafe shutdown count values as a list
        """
        return [self.tool.read_usc(d) for d in dimms]

    def get_dev_dimms(self, dev):
        """
        Get the device's underlying DIMMs. For a particular tool their format
        should be valid as an argument input for all methods.
        """
        return self.tool.get_dev_dimms(dev)


class NdctlUSC(tools.Ndctl, USCTool):
    """
    Ndctl with additional unsafe shutdown handling methods
    implementing USCTool abstract class
    """
    def get_dev_dimms(self, dev):
        region = self._get_dev_region(dev)
        return region['mappings']

    def read_usc(self, dimm):
        try:
            name = dimm['dimm']
        except KeyError as e:
            raise futils.Fail(e)

        out = self._cmd_out_to_json('list', '-HD', '-d', name)
        # dimm's health is nested into a single-element array
        usc = int(out[0]['health']['shutdown_count'])

        return usc



# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2019-2023, Intel Corporation

"""Test framework utilities."""

from os.path import join, abspath
import os
import subprocess as sp

import consts as c
import configurator


def get_tool_path(ctx, name):
    return abspath(join(c.ROOTDIR, '..', 'tools', name, name))


def get_test_tool_path(build, name):
    return abspath(join(c.ROOTDIR, 'tools', name, name))


def get_lib_dir(ctx):
    if str(ctx.build) == 'debug':
        return c.DEBUG_LIBDIR
    else:
        return c.NONDEBUG_LIBDIR


def get_example_path(ctx, libname, name, dirname=None):
    """
    Get the path to the example binary.
    On Unix systems, the example binaries are located in the catalog
    "lib + libname/name" and have the same name as .c file.
    If that is not the case, dirname optional argument can be used to
    specify different catalog for the example binary- "lib + libname/dirname".
    """
    if dirname is None:
        dirname = name
    return abspath(join(c.ROOTDIR, '..', 'examples', 'lib' + libname,
                        dirname, name))


def tail(file, n):
    """
    Replace the file content with the n last lines from the existing file.
    The original file is saved under the name with ".old" suffix.
    """
    with open(file, 'r') as f:
        lines = f.readlines()
        last_lines = lines[-n:]
    os.rename(file, file + ".old")
    with open(file, 'w') as f:
        for line in last_lines:
            f.write(line)


def count(file, substring):
    """
    Count the number of occurrences of a string in the given file.
    """
    with open(file, 'r') as f:
        content = f.read()

    return content.count(substring)


def run_command(cmd, errormsg=""):
    proc = sp.Popen(cmd, shell=True, stdout=sp.PIPE, stderr=sp.STDOUT)
    out = proc.communicate()

    if proc.returncode != 0:
        raise Fail("failed to execute command {}: {}".format(cmd, errormsg))
    return out[0]


class Color:
    """
    Set the font color. This functionality relies on ANSI escape sequences.
    """
    RED = '\33[91m'
    GREEN = '\33[92m'
    YELLOW = '\33[93m'
    END = '\33[0m'


class Message:
    """Simple level based logger."""

    def __init__(self, level):
        self.level = level

    def print(self, msg):
        if self.level >= 1:
            print(msg)

    def print_verbose(self, msg):
        if self.level >= 2:
            print(msg)


class Fail(Exception):
    """Thrown when the test fails."""

    def __init__(self, msg):
        super().__init__(msg)
        self.messages = []
        self.messages.append(msg)

    def __str__(self):
        ret = '\n'.join(self.messages)
        return ret


def fail(msg, exit_code=None):
    if exit_code is not None:
        msg = '{}{}Error {}'.format(msg, os.linesep, exit_code)
    raise Fail(msg)


class Skip(Exception):
    """Thrown when the test should be skipped."""

    def __init__(self, msg):
        super().__init__(msg)
        config = configurator.Configurator().config
        if config.fail_on_skip:
            raise Fail(msg)

        self.messages = []
        self.messages.append(msg)

    def __str__(self):
        ret = '\n'.join(self.messages)
        return ret


def skip(msg):
    raise Skip(msg)


def set_kwargs_attrs(cls, kwargs):
    """
    Translate provided keyword arguments into
    class attributes.
    """
    for k, v in kwargs.items():
        setattr(cls, '{}'.format(k), v)


def add_env_common(src, added):
    """
    A common implementation of adding an environment variable
    to the 'src' environment variables dictionary - taking into account
    that the variable may or may be not already defined.
    """
    for k, v in added.items():
        if k in src:
            src[k] = v + os.pathsep + src[k]
        else:
            src.update({k: v})


def to_list(var, *types):
    """
    Some variables may be provided by the user either as a single instance of
    a type or a sequence of instances (e. g. a string or list of strings).
    To be conveniently treated by the framework code, their types
    should be unified - casted to lists.
    """
    if isinstance(var, tuple(types)):
        return [var, ]
    else:
        return var



#!../env.py
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2021-2024, Intel Corporation
#

import testframework as t


class BASE(t.Test):
    test_type = t.Medium
    niter = 2

    def run(self, ctx):
        ctx.exec('obj_pool_open_mt', ctx.testdir, self.niter)


class TEST0(BASE):
    "multi-threaded open test"


@t.require_valgrind_enabled('helgrind')
class TEST1(BASE):
    "multi-threaded open test /w helgrind"


class TEST2(BASE):
    niter = 8
    "multi-threaded open test /w 8 iterations"



#!../env.py
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2020-2023, Intel Corporation
#

import os

import testframework as t
from testframework import granularity as g


class PMEM2_VM_RESERVATION(t.Test):
    test_type = t.Short
    filesize = 16 * t.MiB
    with_size = True

    def run(self, ctx):
        filepath = ctx.create_holey_file(self.filesize, 'testfile',)
        if self.with_size:
            filesize = os.stat(filepath).st_size
            ctx.exec('pmem2_vm_reservation', self.test_case, filepath,
                     filesize)
        else:
            ctx.exec('pmem2_vm_reservation', self.test_case, filepath)


class PMEM2_VM_RESERVATION_ASYNC(t.Test):
    test_type = t.Short
    filesize = 16 * t.MiB

    def run(self, ctx):
        filepath = ctx.create_holey_file(self.filesize, 'testfile',)
        filesize = os.stat(filepath).st_size
        ctx.exec('pmem2_vm_reservation', self.test_case, filepath,
                 filesize, self.threads, self.ops_per_thread)


@t.require_devdax(t.DevDax('devdax1'))
class PMEM2_VM_RESERVATION_DEVDAX(t.Test):
    test_type = t.Short
    with_size = True

    def run(self, ctx):
        dd = ctx.devdaxes.devdax1
        if self.with_size:
            ctx.exec('pmem2_vm_reservation', self.test_case, dd.path, dd.size)
        else:
            ctx.exec('pmem2_vm_reservation', self.test_case, dd.path)


@t.require_devdax(t.DevDax('devdax1'))
class PMEM2_VM_RESERVATION_ASYNC_DEVDAX(t.Test):
    test_type = t.Short
    with_size = True

    def run(self, ctx):
        dd = ctx.devdaxes.devdax1
        if self.with_size:
            ctx.exec('pmem2_vm_reservation', self.test_case, dd.path, dd.size,
                     self.threads, self.ops_per_thread)
        else:
            ctx.exec('pmem2_vm_reservation', self.test_case, dd.path,
                     self.threads, self.ops_per_thread)


class TEST0(PMEM2_VM_RESERVATION):
    """create a vm reservation with unaligned address provided"""
    test_case = "test_vm_reserv_new_unaligned_addr"


class TEST1(PMEM2_VM_RESERVATION_DEVDAX):
    """DevDax create a vm reservation with unaligned address provided"""
    test_case = "test_vm_reserv_new_unaligned_addr"


class TEST2(PMEM2_VM_RESERVATION):
    """create a vm reservation with unaligned size provided"""
    test_case = "test_vm_reserv_new_unaligned_size"


class TEST3(PMEM2_VM_RESERVATION_DEVDAX):
    """DevDax create a vm reservation with unaligned size provided"""
    test_case = "test_vm_reserv_new_unaligned_size"


class TEST4(PMEM2_VM_RESERVATION):
    """
    map a file to the desired addr  with the help of virtual
    memory reservation
    """
    test_case = "test_vm_reserv_new_valid_addr"


class TEST5(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax map a file to the desired addr  with the help of virtual
    memory reservation
    """
    test_case = "test_vm_reserv_new_valid_addr"


class TEST6(PMEM2_VM_RESERVATION):
    """
    create a vm reservation in the region belonging to other
    existing vm reservation
    """
    test_case = "test_vm_reserv_new_region_occupied_reserv"


class TEST7(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax create a vm reservation in the region belonging to other
    existing vm reservation
    """
    test_case = "test_vm_reserv_new_region_occupied_reserv"


class TEST8(PMEM2_VM_RESERVATION):
    """
    create a vm reservation in the region overlapping whole existing mapping
    """
    test_case = "test_vm_reserv_new_region_occupied_map"


class TEST9(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax create a vm reservation in the region overlapping
    whole existing mapping
    """
    test_case = "test_vm_reserv_new_region_occupied_map"


class TEST10(PMEM2_VM_RESERVATION):
    """
    create a reservation in the region overlapping lower half of
    the existing mapping
    """
    test_case = "test_vm_reserv_new_region_occupied_map_below"


# XXX disable the test for 'pmemcheck', 'drd', 'helgrind', 'memcheck'
# until https://github.com/pmem/pmdk/issues/5592 is fixed.
@t.require_valgrind_disabled('pmemcheck', 'drd', 'helgrind', 'memcheck')
class TEST11(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax create a reservation in the region overlapping lower half of
    the existing mapping
    """
    test_case = "test_vm_reserv_new_region_occupied_map_below"


class TEST12(PMEM2_VM_RESERVATION):
    """
    create a reservation in the region overlapping upper half of
    the existing mapping
    """
    test_case = "test_vm_reserv_new_region_occupied_map_above"


class TEST13(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax create a reservation in the region overlapping upper half of
    the existing mapping
    """
    test_case = "test_vm_reserv_new_region_occupied_map_above"


class TEST14(PMEM2_VM_RESERVATION):
    """create a vm reservation with with error injection"""
    labels = ['fault_injection']
    test_case = "test_vm_reserv_new_alloc_enomem"


class TEST15(PMEM2_VM_RESERVATION_DEVDAX):
    """DevDax create a vm reservation with with error injection"""
    labels = ['fault_injection']
    test_case = "test_vm_reserv_new_alloc_enomem"


class TEST16(PMEM2_VM_RESERVATION):
    """map a file to a vm reservation"""
    test_case = "test_vm_reserv_map_file"


class TEST17(PMEM2_VM_RESERVATION_DEVDAX):
    """DevDax map a file to a vm reservation"""
    test_case = "test_vm_reserv_map_file"


class TEST18(PMEM2_VM_RESERVATION):
    """
    map a part of the file to a vm reservation smaller than the whole file
    """
    test_case = "test_vm_reserv_map_part_file"


class TEST19(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax map a part of the file to a vm reservation
    smaller than the whole file
    """
    test_case = "test_vm_reserv_map_part_file"


class TEST20(PMEM2_VM_RESERVATION):
    """delete a vm reservation that contains a mapping"""
    test_case = "test_vm_reserv_delete_contains_mapping"


class TEST21(PMEM2_VM_RESERVATION):
    """
    map multiple files to a vm reservation, then
    unmap every 2nd mapping and map the mappings again
    """
    test_case = "test_vm_reserv_map_unmap_multiple_files"


# XXX disable the test for 'pmemcheck', 'drd', 'helgrind', 'memcheck'
# until https://github.com/pmem/pmdk/issues/5592 is fixed.
@t.require_valgrind_disabled('pmemcheck', 'drd', 'helgrind', 'memcheck')
class TEST22(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax map multiple files to a vm reservation, then
    unmap every 2nd mapping and map the mappings again
    """
    test_case = "test_vm_reserv_map_unmap_multiple_files"


class TEST23(PMEM2_VM_RESERVATION):
    """map a file to a vm reservation with insufficient space"""
    test_case = "test_vm_reserv_map_insufficient_space"


class TEST24(PMEM2_VM_RESERVATION):
    """
    map a file to a vm reservation and overlap whole other existing mapping
    belonging to the same reservation
    """
    test_case = "test_vm_reserv_map_full_overlap"


class TEST25(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax map a file to a vm reservation and overlap whole other
    existing mapping belonging to the same reservation
    """
    test_case = "test_vm_reserv_map_full_overlap"


class TEST26(PMEM2_VM_RESERVATION):
    """
    map a file to a vm reservation overlapping with the earlier half
    of the other existing mapping
    """
    test_case = "test_vm_reserv_map_partial_overlap_below"


class TEST27(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax map a file to a vm reservation overlapping with the earlier half
    of the other existing mapping
    """
    test_case = "test_vm_reserv_map_partial_overlap_below"


class TEST28(PMEM2_VM_RESERVATION):
    """
    map a file to a vm reservation overlapping with the latter half
    of the other existing mapping
    """
    test_case = "test_vm_reserv_map_partial_overlap_above"


class TEST29(PMEM2_VM_RESERVATION_DEVDAX):
    """
    DevDax map a file to a vm reservation overlapping with the latter half
    of the other existing mapping
    """
    test_case = "test_vm_reserv_map_partial_overlap_above"


@g.require_granularity(g.PAGE, g.CACHELINE)
class TEST30(PMEM2_VM_RESERVATION):
    """
    map a file with invalid granularity to a vm reservation in the middle of
    the vm reservation bigger than the file, then map a file that covers whole
    vm reservation
    """
    test_case = "test_vm_reserv_map_invalid_granularity"


# XXX - https://github.com/pmem/pmdk/issues/5707
@t.DISABLED()
class TEST31(PMEM2_VM_RESERVATION_ASYNC):
    """
    map and unmap asynchronously multiple times to the whole vm reservation
    region
    """
    test_case = "test_vm_reserv_async_map_unmap_multiple_files"
    threads = 32
    ops_per_thread = 1000


# XXX disable the test for 'pmemcheck', 'drd', 'helgrind', 'memcheck'
# until https://github.com/pmem/pmdk/issues/5592 is fixed.
@t.require_valgrind_disabled('pmemcheck', 'drd', 'helgrind', 'memcheck')
class TEST32(PMEM2_VM_RESERVATION_ASYNC_DEVDAX):
    """
    DevDax map and unmap asynchronously multiple times to the whole
    whole vm reservation region
    """
    test_case = "test_vm_reserv_async_map_unmap_multiple_files"
    threads = 32
    ops_per_thread = 1000


class TEST33(PMEM2_VM_RESERVATION):
    """extend the empty vm reservation"""
    test_case = "test_vm_reserv_empty_extend"


class TEST34(PMEM2_VM_RESERVATION):
    """map a file to a vm reservation, extend the reservation and map again"""
    test_case = "test_vm_reserv_map_extend"


class TEST35(PMEM2_VM_RESERVATION):
    """extend the empty vm reservation by unaligned size"""
    test_case = "test_vm_reserv_unaligned_extend"


class TEST36(PMEM2_VM_RESERVATION):
    """
    shrink the empty vm reservation from the start, then from the end,
    lastly map a file to it
    """
    test_case = "test_vm_reserv_empty_shrink"


class TEST37(PMEM2_VM_RESERVATION):
    """
    map a file to the reservation, shrink the reservation from the start,
    then from the end
    """
    test_case = "test_vm_reserv_map_shrink"


class TEST38(PMEM2_VM_RESERVATION):
    """
    shrink the empty vm reservation with unaligned offset,
    then with unaligned size
    """
    test_case = "test_vm_reserv_unaligned_shrink"


class TEST39(PMEM2_VM_RESERVATION):
    """
    shrink the empty vm reservation by interval (offset, offset + size) that is
    out of available range for the reservation to be shrunk
    """
    test_case = "test_vm_reserv_out_of_range_shrink"


class TEST40(PMEM2_VM_RESERVATION):
    """
    shrink the empty vm reservation from the middle, then try shrinking
    reservation by its whole range
    """
    test_case = "test_vm_reserv_unsupported_shrink"


class TEST41(PMEM2_VM_RESERVATION):
    """shrink the vm reservation by the region that is occupied"""
    test_case = "test_vm_reserv_occupied_region_shrink"


class TEST42(PMEM2_VM_RESERVATION):
    """
    create a reservation with exactly the size of a file and map a file to it,
    search for the mapping with the following intervals (offset, size):
    1. (reserv_start, reserv_middle), 2. (reserv_middle, reserv_end),
    3. (reserv_start, reserv_end)
    """
    test_case = "test_vm_reserv_one_map_find"


class TEST43(PMEM2_VM_RESERVATION):
    """
    create a reservation with exactly the size of a 2x file size and map
    a file to it two times, occupying the whole reservation, search for the
    mapping with the following intervals (offset, size):
    1. (reserv_start, reserv_middle), 2. (reserv_middle, reserv_end),
    3. (reserv_start, reserv_end)
    """
    test_case = "test_vm_reserv_two_maps_find"


class TEST44(PMEM2_VM_RESERVATION):
    """
    create a reservation with exactly the size of a 10x file size and map a
    file to it 5 times leaving equal space between each mapping, search the
    reservation for previous mapping for each mapping
    """
    test_case = "test_vm_reserv_prev_map_find"


class TEST45(PMEM2_VM_RESERVATION):
    """
    create a reservation with exactly the size of a 10x file size and map a
    file to it 5 times leaving equal space between each mapping, search the
    reservation for next mapping for each mapping
    """
    test_case = "test_vm_reserv_next_map_find"


class TEST46(PMEM2_VM_RESERVATION):
    """
    create a reservation with exactly the size of a 10x file size and map a
    file to it 5 times leaving equal space between each mapping, search the
    reservation for next mapping for each mapping
    """
    test_case = "test_vm_reserv_not_existing_prev_next_map_find"


class TEST47(PMEM2_VM_RESERVATION):
    """
    create a reservation with exactly the size of 1 file size and map a file to
    it, search for the first and last mapping in the reservation
    """
    test_case = "test_vm_reserv_same_first_last_map_find"


class TEST48(PMEM2_VM_RESERVATION):
    """
    create a reservation with exactly the size of 10 file size and map a file
    10 times to it, search for the first and last mapping in the reservation
    and delete them, repeat until no mappings are left
    """
    test_case = "test_vm_reserv_first_last_map_find"



#!../env.py
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2019-2023, Intel Corporation
#


import testframework as t


class NormalFile(t.Test):
    test_type = t.Short

    def run(self, ctx):
        filepath = ctx.create_holey_file(self.size, 'testfile')
        ctx.exec('pmem2_source_size', self.test_case,
                 filepath, self.size)


class TEST0(NormalFile):
    test_case = 'test_normal_file_fd'
    size = 0


class TEST2(NormalFile):
    test_case = 'test_normal_file_fd'
    size = 16 * t.MiB


class TEST4(t.Test):
    test_type = t.Short

    def run(self, ctx):
        ctx.exec('pmem2_source_size', 'test_tmpfile_fd',
                 ctx.testdir, 16 * t.MiB)


@t.require_devdax(t.DevDax('devdax1'))
class TEST6(t.Test):
    test_type = t.Short

    def run(self, ctx):
        dd = ctx.devdaxes.devdax1
        ctx.exec('pmem2_source_size',
                 'test_normal_file_fd', dd.path, dd.size)



#!../env.py
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2019-2024, Intel Corporation

"""
unit tests for pmemobj_tx_add_range_direct
and pmemobj_tx_xadd_range_direct
"""

from os import path

import testframework as t


@t.require_valgrind_disabled('pmemcheck', 'memcheck')
class TEST0(t.Test):
    test_type = t.Medium

    def run(self, ctx):
        testfile = path.join(ctx.testdir, 'testfile0')
        ctx.exec('obj_tx_add_range_direct', testfile)


@t.require_valgrind_enabled('pmemcheck')
class TEST1(t.Test):
    test_type = t.Medium

    def run(self, ctx):
        ctx.valgrind.add_opt('--mult-stores=no')

        testfile = path.join(ctx.testdir, 'testfile1')
        ctx.exec('obj_tx_add_range_direct', testfile)


@t.require_valgrind_enabled('memcheck')
@t.require_build('debug')
class TEST2(t.Test):
    test_type = t.Medium

    def run(self, ctx):
        testfile = path.join(ctx.testdir, 'testfile2')
        ctx.exec('obj_tx_add_range_direct', testfile)



#!../env.py
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2021-2023, Intel Corporation
#
import os
import testframework as t
import futils

TEST_PARAMS = ['hashmap_tx', 'hashmap_atomic', 'hashmap_rp',
               'ctree', 'btree', 'rtree',
               'rbtree', 'skiplist']


@t.require_build('debug')
@t.require_command('gdb')
class EX_LIBPMEMOBJ(t.Test):
    test_type = t.Medium
    input = ['i 3', 'q']

    def _run_gdb(self, example_path,
                 program_args, b_fun_name):
        gdb_process = self.ctx.gdb(self.testnum, example_path,
                                   program_args, '--batch')
        gdb_process.add_command('set breakpoint pending on')
        gdb_process.add_command(' '.join(['b', b_fun_name]))
        gdb_process.add_command('run')
        gdb_process.add_command('quit')
        gdb_process.execute()


# Test for exit before POBJ_ZNEW
@t.add_params('parameters', TEST_PARAMS)
class TEST26(EX_LIBPMEMOBJ):
    def run(self, ctx):
        example_path = futils.get_example_path(ctx, 'pmemobj',
                                               'mapcli', dirname='map')
        testfile = os.path.join(ctx.testdir, 'testfile1')
        program_args = ' '.join([ctx.parameters(), testfile])
        fun_name = 'map_create'

        self._run_gdb(example_path, program_args, fun_name)
        ctx.exec(example_path, ctx.parameters(),
                 testfile, std_input=self.input)


# Test for exit after POBJ_ZNEW
@t.add_params('parameters', TEST_PARAMS)
class TEST27(EX_LIBPMEMOBJ):
    def run(self, ctx):
        example_path = futils.get_example_path(ctx, 'pmemobj',
                                               'mapcli', dirname='map')
        testfile = os.path.join(ctx.testdir, 'testfile1')
        arg_fun_dict = {'hashmap_tx': 'create_hashmap',
                        'hashmap_atomic': "create_hashmap",
                        'hashmap_rp': "hashmap_create",
                        'ctree': 'ctree_map_create',
                        'btree': 'btree_map_create',
                        'rtree': 'rtree_map_create',
                        'rbtree': 'rbtree_map_create',
                        'skiplist': 'skiplist_map_create'}
        program_args = ' '.join([ctx.parameters(), testfile])
        fun_name = arg_fun_dict.get(ctx.parameters())

        self._run_gdb(example_path, program_args, fun_name)
        ctx.exec(example_path, ctx.parameters(),
                 testfile, std_input=self.input)



#!/usr/bin/env python3

# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2023, Intel Corporation

import argparse
import json

from typing import Dict, Any

# https://peps.python.org/pep-0589/ requires Python >= 3.8
# from typing import TypedDict

# Print out stack usage per function call for a given call stack.

PARSER = argparse.ArgumentParser()
PARSER.add_argument('-s', '--stack-usage-file', default='stack_usage.json')
PARSER.add_argument('-c', '--call-stack', required=True)

# class StackUsageRecord(TypedDict): # for Python >= 3.8
#     size: int
StackUsageRecord = Dict[str, int] # for Python < 3.8

# class CallStack(TypedDict): # for Python >= 3.8
#     stack: list[str]
#     size: int
CallStack = Dict[str, Any] # for Python < 3.8

def load_stack_usage(stack_usage_file: str) -> Dict[str, StackUsageRecord]:
        with open(stack_usage_file, 'r') as file:
                return json.load(file)

def load_call_stack(call_stack: str) -> CallStack:
        with open(call_stack, 'r') as file:
                return json.load(file)

def main():
        args = PARSER.parse_args()
        stack_usage = load_stack_usage(args.stack_usage_file)
        call_stack = load_call_stack(args.call_stack)
        for func in call_stack['stack']:
                if func in stack_usage.keys():
                        size = stack_usage[func]['size']
                else:
                        size = 0
                print(f"{size}\t{func}")

if __name__ == '__main__':
        main()



#!../env.py
# SPDX-License-Identifier: BSD-3-Clause
# Copyright 2019-2024, Intel Corporation
#

from os import path
import testframework as t


class BASE(t.Test):
    test_type = t.Medium

    def run(self, ctx):
        testfile = path.join(ctx.testdir, 'testfile0')
        ctx.exec('obj_defrag', testfile)


class TEST0(BASE):
    "defrag test"
